/*======================PROCEDIMENTO ORCAMENTO_OK===============================

   PROCEDIMENTO VERIFICA SE O VALOR DE CONTA (A PAGAR OU A RECEBER) NO ULTRAPASSA
   O VALOR ORADO PARA O MS E CLASSIFICAO FINANCEIRA DA EMPRESA. SE NO ULTRAPASSA,
   ATUALIZA O VALOR DO ORAMENTO RESTANTE PARA O MS

   -------
   ENTRADA
   -------
      O PROCEDIMENTO RECEBE OS SEGUINTES PARMETROS :

      - P_CDEMPRESA    : CDIGO DA EMPRESA DA CONTA
      - P_CDFILIAL     : CDIGO DA FILIAL DE ORIGEM DA CONTA
      - P_CDCLASSFINA  : CDIGO DA CLASSIFICAO FINANCEIRA DO TIPO DE CONTA
      - P_DTLANCAMENTO : DATA DE VENCIMENTO DA CONTA
      - P_VRVALOR      : VALOR DA CONTA
   -----
   SADA
   -----
      O RETORNO DO PROCEDIMENTO SE D PELO PARMETRO:
      - P_RESULTADO    : 0 - ORAMENTO FOI ULTRAPASSADO PELO VALOR DA CONTA
                         1 - ORAMENTO OK
*/
IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'ORCAMENTO_OK' AND
                 TYPE = 'P' 		       AND
	              U.UID = O.UID            AND
	              U.NAME = USER)
DROP PROCEDURE ORCAMENTO_OK
GO

CREATE PROCEDURE ORCAMENTO_OK(@P_CDEMPRESA VARCHAR(2),
                              @P_CDFILIAL  VARCHAR(4),
                              @P_CDCLASSFINA VARCHAR(6),
                              @P_DTLANCAMENTO DATETIME,
                              @P_VRVALOR NUMERIC(15,2),
                              @P_RESULTADO FLOAT OUTPUT)
AS
   DECLARE @V_IDCONSORCFIN VARCHAR(1);
   DECLARE @V_IDTPORCAFINA VARCHAR(1);
   DECLARE @V_DTINIMES DATETIME;
   DECLARE @V_DTFINMES DATETIME;
   DECLARE @V_CDFILIAL VARCHAR(4);
   DECLARE @V_ORC_CDFILIAL VARCHAR(4);
   DECLARE @V_ORC_CDCLASSFINA VARCHAR(6);
   DECLARE @V_ORC_DTORCAFINA DATETIME;
   DECLARE @V_ORC_VRORCAFINA NUMERIC(15,2);
   DECLARE @V_ORC_VREXTOLEORCA NUMERIC(15,2);
   DECLARE @V_ORC_VRREALORCA NUMERIC(15,2);
BEGIN
   SELECT @V_IDCONSORCFIN = IDCONSORCFIN,
          @V_IDTPORCAFINA = IDTPORCAFINA
     FROM PARAMGERAL;
   /*VERIFICA SE CONSISTE OU NO ORAMENTO FINANCEIRO*/
   IF (@V_IDCONSORCFIN = 'S') AND (@P_VRVALOR <> 0) BEGIN
      /*VERIFICA SE O TIPO DE ORAMENTO  POR FILIAL*/
      IF @V_IDTPORCAFINA = 'F' BEGIN
         SET @V_CDFILIAL = @P_CDFILIAL;
      END
      ELSE BEGIN
         SET @V_CDFILIAL = NULL;
      END;
      SET @V_DTINIMES = CONVERT(DATETIME, '01' + '/' +
      RIGHT(REPLICATE('0', 2) + CONVERT(VARCHAR,MONTH(@P_DTLANCAMENTO)), 2) + '/' +
      RIGHT(REPLICATE('0', 4) + CONVERT(VARCHAR,YEAR(@P_DTLANCAMENTO)), 4), 103);
      SET @V_DTFINMES = DATEADD(MM,1,@P_DTLANCAMENTO)-DAY(@P_DTLANCAMENTO);
      /*PESQUISA O ORAMENTO FINANCEIRO PARA A EMPRESA, CLASSIFICAO FINANCEIRA,
      FILIAL E MES DA DATA DE LANAMENTO*/
      DECLARE C_ORCAFIN CURSOR FOR
         SELECT CDFILIAL,
                CDCLASSFINA,
                DTORCAFINA,
                ISNULL(VRORCAFINA, 0) VRORCAFINA,
                ISNULL(VREXTOLEORCA, 0) VREXTOLEORCA,
                ISNULL(VRREALORCA, 0) VRREALORCA
           FROM ORCAFINA
          WHERE (CDEMPRESA   = @P_CDEMPRESA)   AND
                (CDFILIAL    = ISNULL(@V_CDFILIAL, CDFILIAL)) AND
                (CDCLASSFINA = @P_CDCLASSFINA) AND
                (DTORCAFINA BETWEEN @V_DTINIMES AND @V_DTFINMES);
      OPEN C_ORCAFIN;
      FETCH NEXT FROM C_ORCAFIN INTO
      @V_ORC_CDFILIAL,
      @V_ORC_CDCLASSFINA,
      @V_ORC_DTORCAFINA,
      @V_ORC_VRORCAFINA,
      @V_ORC_VREXTOLEORCA,
      @V_ORC_VRREALORCA;

      WHILE (@@FETCH_STATUS = 0) BEGIN
         /*VERIFICA SE O VALOR ORADO SER ULTRAPASSADO*/
         IF ROUND(@V_ORC_VRREALORCA + @P_VRVALOR, 2) > ROUND(@V_ORC_VRORCAFINA + @V_ORC_VREXTOLEORCA, 2) BEGIN
            SET @P_RESULTADO = 0;
            RETURN;
         END;
         /*ATUALIZA O VALOR DO ORAMENTO J UTILIZADO*/
         UPDATE ORCAFINA
            SET VRREALORCA  = ROUND(@V_ORC_VRREALORCA + @P_VRVALOR, 2)
          WHERE CDEMPRESA   = @P_CDEMPRESA AND
                CDCLASSFINA  = @P_CDCLASSFINA AND
                DTORCAFINA   = @V_ORC_DTORCAFINA;
         FETCH NEXT FROM C_ORCAFIN INTO
         @V_ORC_CDFILIAL,
         @V_ORC_CDCLASSFINA,
         @V_ORC_DTORCAFINA,
         @V_ORC_VRORCAFINA,
         @V_ORC_VREXTOLEORCA,
         @V_ORC_VRREALORCA;
      END;
      CLOSE C_ORCAFIN;
      DEALLOCATE C_ORCAFIN;
   END;
   SET @P_RESULTADO = 1;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'ORCAMENTO_OK','5.11.000';
GO